@@ -19,13 +19,27 @@
 
 #include "Program.h"
 
+// LLVM
+#include <llvm/DerivedTypes.h>
+#include <llvm/Instructions.h>
+#include <llvm/Module.h>
+#include <llvm/Transforms/Utils/Cloning.h>
+
+// Passes
+#include <llvm/PassManager.h>
+#include <llvm/Analysis/LoopPass.h>
+#include <llvm/Analysis/Verifier.h>
+#include <llvm/Target/TargetData.h>
+#include <llvm/ExecutionEngine/ExecutionEngine.h>
+#include <llvm/GlobalVariable.h>
+
 // GTLCore
 #include "GTLCore/Buffer.h"
 #include "GTLCore/BufferImage.h"
-#include "GTLCore/CodeGenerator_p.h"
-#include "GTLCore/GenerationContext_p.h"
+#include "GTLCore/LLVMBackend/CodeGenerator_p.h"
+#include "GTLCore/LLVMBackend/GenerationContext_p.h"
 #include "GTLCore/PixelDescription.h"
-#include "GTLCore/ExpressionResult_p.h"
+#include "GTLCore/LLVMBackend/ExpressionResult_p.h"
 #include "GTLCore/Function.h"
 #include "GTLCore/Function_p.h"
 #include "GTLCore/Macros_p.h"
@@ -42,22 +56,6 @@
 #include "GTLCore/ModuleData_p.h"
 #include "Debug.h"
 #include "GTLCore/VirtualMachine_p.h"
-#include "GTLCore/Optimiser_p.h"
-
-// LLVM
-#include <llvm/DerivedTypes.h>
-#include <llvm/Instructions.h>
-#include <llvm/Module.h>
-#include <llvm/ModuleProvider.h>
-#include <llvm/Transforms/Utils/Cloning.h>
-
-// Passes
-#include <llvm/PassManager.h>
-#include <llvm/Analysis/LoopPass.h>
-#include <llvm/Analysis/Verifier.h>
-#include <llvm/Target/TargetData.h>
-#include <llvm/ExecutionEngine/ExecutionEngine.h>
-#include <llvm/GlobalVariable.h>
 
 using namespace OpenCTL;
 
@@ -72,8 +70,7 @@ struct Program::Private {
   }
   llvm::Module* module;
   GTLCore::ModuleData* moduleData;
-  void (*func)( const char**, char*, int);
-  llvm::ModuleProvider* moduleProvider;
+  void (*func)( const char**, char*, int, GTLCore::ProgressReport*, gtl_uint64);
   std::list<GTLCore::PixelDescription> srcPixelDescriptions;
   GTLCore::PixelDescription dstPixelDescription;
   static int s_id;
@@ -84,9 +81,9 @@ struct Program::Private {
 
 int Program::Private::s_id = 0;
 
-void configureBuffer(GTLCore::CodeGenerator& cg, const std::vector< const GTLCore::Type*>& channelsTypes, llvm::Value**& indexes, bool*& needBuffer, int countChannels, llvm::LLVMContext& context)
+void configureBuffer(LLVMBackend::CodeGenerator& cg, const std::vector< const GTLCore::Type*>& channelsTypes, llvm::Value**& indexes, bool*& needBuffer, int countChannels, llvm::LLVMContext& context)
 {
-  int currentPos = 0;
+  gtl_int32 currentPos = 0;
   for( int i = 0; i < countChannels; ++i)
   {
     indexes[i] = cg.integerToConstant( context, currentPos );
@@ -96,7 +93,7 @@ void configureBuffer(GTLCore::CodeGenerator& cg, const std::vector< const GTLCor
         or channelsTypes[i] == GTLCore::Type::UnsignedInteger8
         or channelsTypes[i] == GTLCore::Type::Integer16
         or channelsTypes[i] == GTLCore::Type::UnsignedInteger16
-        or channelsTypes[i] == GTLCore::Type::Half )
+        or channelsTypes[i] == GTLCore::Type::Float16 )
     {
       needBuffer[i] = true;
     } else {
@@ -127,19 +124,19 @@ struct InputGenInfo {
     pixelSize = _pd.bitsSize() / 8;
     OCTL_ASSERT(_pd.bitsSize() % 8== 0);
     indexes = new llvm::Value*[ channelsTypes.size() ];
-    needBuffer = new bool[ channelsTypes.size() ]; // CTL doesn't support Int8 or Int16 (and OpenCTL's VM doesn't know about half), so when some pixel channel is in Int8 or Int16 (or half) it first need to be converted to Int32 (or float)
+    needConversion = new bool[ channelsTypes.size() ]; // CTL doesn't support Int8 or Int16 (and OpenCTL's VM doesn't know about half), so when some pixel channel is in Int8 or Int16 (or half) it first need to be converted to Int32 (or float)
     countChannels = channelsTypes.size();
-    pixelSizeValue = GTLCore::CodeGenerator::integerToConstant( context, pixelSize );
+    pixelSizeValue = LLVMBackend::CodeGenerator::integerToConstant( context, pixelSize );
   }
   ~InputGenInfo() {
     delete[] indexes;
-    delete[] needBuffer;
+    delete[] needConversion;
     delete pos;
   }
   const std::vector< const GTLCore::Type*>& channelsTypes;
-  int pixelSize;
+  gtl_int32 pixelSize;
   llvm::Value** indexes;
-  bool* needBuffer;
+  bool* needConversion;
   llvm::Value* pixelSizeValue;
   int countChannels;
   GTLCore::VariableNG* pos;
@@ -152,7 +149,6 @@ struct InputGenInfo {
 void Program::init(const GTLCore::String& functionName, const Module* module)
 {
   d->func = 0;
-  d->moduleProvider = 0;
   d->moduleData = 0;
   // Clone the module
   if( module->data())
@@ -175,11 +171,10 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
     const GTLCore::Function* functionDef = module->function( functionName, arguments );
     if( not functionDef ) return;
     
-    
-    llvm::Function* function = d->module->getFunction( GTLCore::Function::Data::symbolName( GTLCore::ScopedName( "", functionName), functionDef->returnType(),functionDef->parameters() ) );
+    llvm::Function* function = d->module->getFunction( (const std::string&)GTLCore::Function::Data::symbolName( GTLCore::ScopedName( "", functionName), functionDef->returnType(),functionDef->parameters() ) );
     if(function)
     {
-      GTLCore::CodeGenerator cg( d->moduleData );
+      LLVMBackend::CodeGenerator cg( d->moduleData );
       // Initialise pixel information and Configure input buffers
       OCTL_DEBUG("Configure input buffer");
       std::vector<InputGenInfo*> srcs;
@@ -188,17 +183,17 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
       {
         InputGenInfo* src = new InputGenInfo(*it, d->module->getContext());
         srcs.push_back( src);
-        configureBuffer( cg, src->channelsTypes, src->indexes, src->needBuffer, src->countChannels, d->module->getContext());
+        configureBuffer( cg, src->channelsTypes, src->indexes, src->needConversion, src->countChannels, d->module->getContext());
       }
       // Initialise pixel information and Configure output buffers
       InputGenInfo dst( d->dstPixelDescription, d->module->getContext());
-      configureBuffer( cg, dst.channelsTypes, dst.indexes, dst.needBuffer, dst.countChannels, d->module->getContext());
+      configureBuffer( cg, dst.channelsTypes, dst.indexes, dst.needConversion, dst.countChannels, d->module->getContext());
       
       //---------- Program-Pseudo-code ----------//
       // This just a rough explanation of the function that get generated here.
       //
       // template<int pixelSize, typename _TYPE_>
-      // void program(const char** ins8, char* out8, int size)
+      // void program(const char** ins8, char* out8, int size, ProgressReport* _report)
       // {
       //   _BUFFER_TYPE_ buffer[pixelSize]; // buffer for Int8, Int16 and half channels
       //   for(int i = 0; i < size; i += pixelSize)
@@ -211,20 +206,23 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
       //       
       //     }
       //     function(in0[i], in0[i+indexes[1]] ..., out + i, out + i +indexes[1], ...);
+      //     _report->nextPixel();
       //   }
       //   return;
       // }
       OCTL_DEBUG("Initialize the function type");
       std::vector<const llvm::Type*> params;
-      params.push_back( llvm::PointerType::get( llvm::PointerType::get( llvm::Type::getInt8Ty(d->module->getContext()), 0 ), 0 ) );
-      params.push_back( llvm::PointerType::get( llvm::Type::getInt8Ty(d->module->getContext()), 0 ));
-      params.push_back( llvm::Type::getInt32Ty(d->module->getContext()) );
+      params.push_back( llvm::PointerType::get( llvm::PointerType::get( llvm::Type::getInt8Ty(d->module->getContext()), 0 ), 0 ) ); // const char** int8
+      params.push_back( llvm::PointerType::get( llvm::Type::getInt8Ty(d->module->getContext()), 0 )); // char* out8
+      params.push_back( llvm::Type::getInt32Ty(d->module->getContext()) ); // int size
+      params.push_back( GTLCore::Type::Pointer->d->type(d->module->getContext()) ); // ProgressReport* _report
+      params.push_back( llvm::Type::getInt64Ty(d->module->getContext()) );
       // void program(const char* in8, char* out8, int size)
       llvm::FunctionType* definitionType = llvm::FunctionType::get( llvm::Type::getVoidTy(d->module->getContext()), params, false );
       int programNb = ++Program::Private::s_id;
-      llvm::Function* func = cg.createFunction( definitionType, "CTLProgram" + GTLCore::String::number(programNb));
+      llvm::Function* func = cg.createFunction( d->module, definitionType, "CTLProgram" + GTLCore::String::number(programNb));
       // Initialise a generation context
-      GTLCore::GenerationContext gc( &cg, &d->module->getContext(), func, 0, d->moduleData );
+      LLVMBackend::GenerationContext gc( &cg, &d->module->getContext(), func, 0, d->moduleData, d->module );
       // {
       OCTL_DEBUG("Initial block");
       llvm::BasicBlock* initialBlock = llvm::BasicBlock::Create(d->module->getContext());
@@ -234,20 +232,20 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
       GTLCore::VariableNG** buffer = new GTLCore::VariableNG*[dst.countChannels];
       for(int i = 0; i < dst.countChannels; ++i)
       {
-        if(dst.needBuffer[i])
+        GTLCore::VariableNG* vng = 0;
+        if(dst.needConversion[i])
         {
-          GTLCore::VariableNG* vng = 0;
-          if(dst.channelsTypes[i] == GTLCore::Type::Half)
+          if(dst.channelsTypes[i] == GTLCore::Type::Float16)
           {
-            vng = new GTLCore::VariableNG(GTLCore::Type::Float, false, false );
+            vng = new GTLCore::VariableNG(GTLCore::Type::Float32, false, false );
           } else {
             vng = new GTLCore::VariableNG(GTLCore::Type::Integer32, false, false );
           }
-          vng->initialise( gc, initialBlock, GTLCore::ExpressionResult(), std::list<llvm::Value*>());
-          buffer[i] = vng;
         } else {
-          buffer[i] = 0;
+          vng = new GTLCore::VariableNG(dst.channelsTypes[i], false, false );
         }
+        vng->initialise( gc, initialBlock, LLVMBackend::ExpressionResult(), std::list<llvm::Value*>());
+        buffer[i] = vng;
       }
       OCTL_DEBUG("Get the arguments");
       // Get the args.
@@ -257,14 +255,21 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
       
       for(std::size_t k = 0; k < srcs.size(); ++k)
       {
-        srcs[k]->addr = new llvm::LoadInst( llvm::GetElementPtrInst::Create( ins, cg.integerToConstant(d->module->getContext(), k), "", initialBlock ), "", initialBlock );
+        srcs[k]->addr = new llvm::LoadInst( llvm::GetElementPtrInst::Create( ins, cg.integerToConstant(d->module->getContext(), gtl_uint32(k)), "", initialBlock ), "", initialBlock );
       }
       //   char* out8 = second arg;
       ++arg_it;
       llvm::Value* out = arg_it;
-      // size
+      // size = third arg
       ++arg_it;
       llvm::Value* size = arg_it;
+      //   void* _report = fourth arg;
+      ++arg_it;
+      llvm::Value* arg_report = arg_it;
+      //   void* _channelFlags = fifth arg;
+      ++arg_it;
+      llvm::Value* arg_channelFlags = arg_it;
+      
     // Construct the "conditions" of the loop
       
       //   int i = 0;
@@ -273,10 +278,10 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
       for(std::size_t k = 0; k < srcs.size(); ++k)
       {
         srcs[k]->pos = new GTLCore::VariableNG( GTLCore::Type::Integer32, false, false );
-        srcs[k]->pos->initialise( gc, initialBlock, GTLCore::ExpressionResult(cg.integerToConstant(d->module->getContext(), 0), GTLCore::Type::Integer32), std::list<llvm::Value*>());
+        srcs[k]->pos->initialise( gc, initialBlock, LLVMBackend::ExpressionResult(cg.integerToConstant(d->module->getContext(), INT32_C(0)), GTLCore::Type::Integer32), std::list<llvm::Value*>());
       }
       dst.pos = new GTLCore::VariableNG( GTLCore::Type::Integer32, false, false );
-      dst.pos->initialise( gc, initialBlock, GTLCore::ExpressionResult(cg.integerToConstant(d->module->getContext(), 0), GTLCore::Type::Integer32), std::list<llvm::Value*>());
+      dst.pos->initialise( gc, initialBlock, LLVMBackend::ExpressionResult(cg.integerToConstant(d->module->getContext(), INT32_C(0)), GTLCore::Type::Integer32), std::list<llvm::Value*>());
       
       // i < size
       OCTL_DEBUG("i < size");
@@ -297,6 +302,7 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
       // Construct the body of the for loop
       OCTL_DEBUG("bodyBlock");
       llvm::BasicBlock* bodyBlock = llvm::BasicBlock::Create(d->module->getContext(), "bodyBlock");
+      llvm::BasicBlock* startBodyBlock = bodyBlock;
       func->getBasicBlockList().push_back( bodyBlock);
       
       // function(in[i], in[i+indexes[1]] ..., out + i, out + i +indexes[1], ...);
@@ -315,14 +321,14 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
                         cg.convertPointerTo( bodyBlock,
                                   llvm::GetElementPtrInst::Create( srcs[k]->addr, cg.createAdditionExpression( bodyBlock, srcIndex, srcs[k]->pos->type(), srcs[k]->indexes[i], GTLCore::Type::Integer32), "", bodyBlock), srcs[k]->channelsTypes[i]->d->type(d->module->getContext())),
                         "", bodyBlock);
-          if( srcs[k]->needBuffer[i])
+          if( srcs[k]->needConversion[i])
           { // if a buffer is needed that means that the value must be converted
             OCTL_DEBUG("if a buffer is needed that means that the value must be converted");
-            if(srcs[k]->channelsTypes[i] == GTLCore::Type::Half)
+            if(srcs[k]->channelsTypes[i] == GTLCore::Type::Float16)
             {
-              convertedIn = GTLCore::CodeGenerator::convertFromHalf( gc, bodyBlock, convertedIn);
+              convertedIn = LLVMBackend::CodeGenerator::convertFromHalf( gc, bodyBlock, convertedIn);
             } else {
-              convertedIn = GTLCore::CodeGenerator::convertValueTo( bodyBlock, convertedIn, srcs[k]->channelsTypes[i], GTLCore::Type::Integer32);
+              convertedIn = LLVMBackend::CodeGenerator::convertValueTo( bodyBlock, convertedIn, srcs[k]->channelsTypes[i], GTLCore::Type::Integer32);
             }
           }
           arguments.push_back( convertedIn );
@@ -333,15 +339,7 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
       llvm::Value* dstIndex = dst.pos->get( gc, bodyBlock);
       for(int i = 0; i < dst.countChannels; ++i)
       {
-        llvm::Value* pointer = cg.convertPointerTo( bodyBlock, llvm::GetElementPtrInst::Create( out, cg.createAdditionExpression( bodyBlock, dstIndex, GTLCore::Type::Integer32, dst.indexes[i], GTLCore::Type::Integer32), "", bodyBlock), d->dstPixelDescription.channelTypes()[i]->d->type(d->module->getContext()));
-        if( dst.needBuffer[i])
-        {
-          OCTL_ASSERT(buffer[i]);
-          buffer[i]->set( gc, bodyBlock, new llvm::LoadInst(pointer,"", bodyBlock), d->dstPixelDescription.channelTypes()[i]);
-          arguments.push_back( buffer[i]->pointer(bodyBlock));
-        } else {
-          arguments.push_back(pointer);
-        }
+        arguments.push_back( buffer[i]->pointer(bodyBlock));
       }
       // Check if there are more parameters to call
       OCTL_DEBUG("Check if there are more parameters to call");
@@ -356,7 +354,7 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
                     *d->module,
                     parameters[i].type()->d->type(d->module->getContext()),
                     false, llvm::GlobalValue::ExternalLinkage,
-                    GTLCore::CodeGenerator::valueToConstant( gc, parameters[i].defaultValue() ), "" );
+                    LLVMBackend::CodeGenerator::valueToConstant( gc, parameters[i].defaultValue() ), "" );
           arguments.push_back( new llvm::LoadInst( globalVar, "", bodyBlock ) );
           d->varyings[ parameters[i].name() ] = globalVar;
           d->varyingsName.push_back( parameters[i].name() );
@@ -377,28 +375,56 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
       OCTL_DEBUG("If there was buffering, save to output");
       for(int i = 0; i < dst.countChannels; ++i)
       {
-        if( dst.needBuffer[i])
+        
+        llvm::BasicBlock* ifBlock = llvm::BasicBlock::Create(gc.llvmContext());
+        func->getBasicBlockList().push_back( ifBlock );
+        
+        const llvm::Type* channelType = dst.channelsTypes[i]->d->type(d->module->getContext());
+        llvm::Value* pointer = cg.convertPointerTo( ifBlock, llvm::GetElementPtrInst::Create( out, cg.createAdditionExpression( ifBlock, dstIndex, GTLCore::Type::Integer32, dst.indexes[i], GTLCore::Type::Integer32), "", ifBlock), channelType);
+        llvm::Value* result = buffer[i]->get( gc, ifBlock );
+        if( dst.needConversion[i])
         {
-          const llvm::Type* channelType = dst.channelsTypes[i]->d->type(d->module->getContext());
-          llvm::Value* pointer = cg.convertPointerTo( bodyBlock, llvm::GetElementPtrInst::Create( out, cg.createAdditionExpression( bodyBlock, dstIndex, GTLCore::Type::Integer32, dst.indexes[i], GTLCore::Type::Integer32), "", bodyBlock), channelType);
-          llvm::Value* result = 0;
-          if( dst.channelsTypes[i] == GTLCore::Type::Half )
+          if( dst.channelsTypes[i] == GTLCore::Type::Float16 )
           {
-            result = GTLCore::CodeGenerator::convertToHalf( gc, bodyBlock, buffer[i]->get( gc, bodyBlock ), GTLCore::Type::Float );
+            result = LLVMBackend::CodeGenerator::convertToHalf( gc, ifBlock, result, GTLCore::Type::Float32 );
           } else {
-            result = GTLCore::CodeGenerator::convertValueTo( bodyBlock, buffer[i]->get( gc, bodyBlock ), GTLCore::Type::Integer32, dst.channelsTypes[i] );
+            result = LLVMBackend::CodeGenerator::convertValueTo( ifBlock, result, GTLCore::Type::Integer32, dst.channelsTypes[i] );
           }
-          OCTL_DEBUG( *result->getType() << *pointer->getType() );
-          new llvm::StoreInst( result, pointer, bodyBlock);
         }
+        
+        //      gtl_uint64 channelMask = 1 << k;
+        gtl_uint64 channelMask = gtl_uint64(1) << i;
+        llvm::Value* channelMaskValue = LLVMBackend::CodeGenerator::valueToConstant(gc, channelMask );
+        // (_channelFlags & channelMask)
+        llvm::Value* lhs = LLVMBackend::CodeGenerator::createBitAndExpression(bodyBlock,
+                                  arg_channelFlags, GTLCore::Type::UnsignedInteger64,
+                                  channelMaskValue, GTLCore::Type::UnsignedInteger64 );
+        
+        //  (_channelFlags & channelMask) == channelMask
+        llvm::Value* comp = LLVMBackend::CodeGenerator::createEqualExpression(bodyBlock,
+                                  lhs, GTLCore::Type::UnsignedInteger64,
+                                  channelMaskValue, GTLCore::Type::UnsignedInteger64);
+        
+        //      if( (_channelFlags & channelMask) == channelMask )
+        llvm::BasicBlock* nextCurrentBlock = llvm::BasicBlock::Create(gc.llvmContext());
+        func->getBasicBlockList().push_back( nextCurrentBlock );
+          
+        LLVMBackend::CodeGenerator::createIfStatement(bodyBlock, comp, GTLCore::Type::Boolean, ifBlock, ifBlock, nextCurrentBlock);
+
+        bodyBlock = nextCurrentBlock;
+        
+        OCTL_DEBUG( *result->getType() << *pointer->getType() );
+        new llvm::StoreInst( result, pointer, bodyBlock);
       }
+    // Call the progress report
+      bodyBlock = LLVMBackend::CodeGenerator::callProgressReportNextPixel( gc, arg_report, bodyBlock );
     // Put the for loop together
       OCTL_DEBUG("Put the for loop together");
       // for(int i = 0; i < size; ++i)
       OCTL_DEBUG("for(int i = 0; i < size; ++i)");
       llvm::BasicBlock* finBlock = llvm::BasicBlock::Create(d->module->getContext(), "finBlock");
       func->getBasicBlockList().push_back( finBlock);
-      cg.createForStatement(initialBlock, forTestBlock, forTestBlock, forTest, GTLCore::Type::Boolean, updateBlock, bodyBlock, bodyBlock, finBlock);
+      cg.createForStatement(initialBlock, forTestBlock, forTestBlock, forTest, GTLCore::Type::Boolean, updateBlock, startBodyBlock, bodyBlock, finBlock);
       // return;
       OCTL_DEBUG("return;");
       llvm::ReturnInst::Create(d->module->getContext(), finBlock);
@@ -407,10 +433,9 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
       OCTL_DEBUG("Optimize");
       
       // Register module in the VM
-      d->moduleProvider = new llvm::ExistingModuleProvider( d->module );
-      GTLCore::VirtualMachine::instance()->registerModule( d->moduleProvider );
+      GTLCore::VirtualMachine::instance()->registerModule( d->module );
       GTLCore::VirtualMachine::instance()->executionEngine()->clearAllGlobalMappings();
-      d->func = ( void(*)(const char**, char*,int)) GTLCore::VirtualMachine::instance()->getPointerToFunction( func );
+      d->func = ( void(*)(const char**, char*,int,GTLCore::ProgressReport*, gtl_uint64)) GTLCore::VirtualMachine::instance()->getPointerToFunction( func );
       for( std::map< GTLCore::String, llvm::GlobalVariable*>::iterator it = d->varyings.begin(); it != d->varyings.end(); ++it)
       {
           d->varyingsPtr[ it->first ] = GTLCore::VirtualMachine::instance()->getGlobalVariablePointer( it->second );
@@ -436,26 +461,20 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
 
 Program::~Program()
 {
-  if(d->moduleProvider)
+  if(d->module)
   {
-    GTLCore::VirtualMachine::instance()->unregisterModule( d->moduleProvider);
-    delete d->moduleProvider;
+    GTLCore::VirtualMachine::instance()->unregisterModule( d->module);
   }
   delete d->moduleData;
   delete d;
 }
 
-bool Program::initialised() const
-{
-  return d->func;
-}
-
 bool Program::isInitialised() const
 {
   return d->func;
 }
 
-void Program::apply(const GTLCore::Buffer& input, GTLCore::Buffer& output) const
+void Program::apply(const GTLCore::Buffer& input, GTLCore::Buffer& output, GTLCore::ProgressReport* _report, const GTLCore::ChannelsFlags& flags ) const
 {
   GTL_ASSERT(d->srcPixelDescriptions.size() == 1);
   OCTL_ASSERT( (input.size() / (d->srcPixelDescriptions.begin()->bitsSize() / 8) ) == (output.size() / (d->dstPixelDescription.bitsSize() / 8) ) );
@@ -465,11 +484,11 @@ void Program::apply(const GTLCore::Buffer& input, GTLCore::Buffer& output) const
   OCTL_DEBUG("Apply program on a buffer of size " << input.size() << " for a source pixel of size " << (d->srcPixelDescriptions.begin()->bitsSize() / 8) << " for a destination pixel of size " << (d->dstPixelDescription.bitsSize() / 8));
   const char * in[1];
   in[0] = input.rawData();
-  d->func( in, output.rawData(), input.size() );
+  d->func( in, output.rawData(), input.size(), _report, flags.value() );
   OCTL_DEBUG("Applied");
 }
 
-void Program::apply(const std::list<GTLCore::Buffer*>& inputs, GTLCore::Buffer& output) const
+void Program::apply(const std::list<GTLCore::Buffer*>& inputs, GTLCore::Buffer& output, GTLCore::ProgressReport* _report, const GTLCore::ChannelsFlags& flags ) const
 {
   GTL_ASSERT(d->srcPixelDescriptions.size() == inputs.size());
   OCTL_ASSERT( (output.size() % (d->dstPixelDescription.bitsSize() / 8)) == 0 );
@@ -481,18 +500,18 @@ void Program::apply(const std::list<GTLCore::Buffer*>& inputs, GTLCore::Buffer&
     ins[idx] = (*it)->rawData();
 //     OCTL_ASSERT( ((*it)->size() / (d->srcPixelDescription.bitsSize() / 8) ) == (output.size() / (d->dstPixelDescription.bitsSize() / 8) ) );
   }
-  d->func(ins, output.rawData(), (*inputs.begin())->size());
+  d->func(ins, output.rawData(), (*inputs.begin())->size(), _report, flags.value());
 }
 
 
-void Program::apply(const GTLCore::AbstractImage& input, GTLCore::AbstractImage& output) const
+void Program::apply(const GTLCore::AbstractImage& input, GTLCore::AbstractImage& output, GTLCore::ProgressReport* report, const GTLCore::ChannelsFlags& flags ) const
 {
   GTLCore::AbstractImage::ConstIterator* itSrc = input.createIterator();
   GTLCore::AbstractImage::Iterator* itDst = output.createIterator();
   
   while( itSrc->next() and itDst->next())
   {
-    apply( *itSrc->current(), *itDst->current() );
+    apply( *itSrc->current(), *itDst->current(), report, flags );
   }
   
   delete itDst;
@@ -514,8 +533,8 @@ void Program::setVarying( const GTLCore::String& _name, const GTLCore::Value& _v
         *(bool*)ptr = _value.asBoolean();
     } else if( it2->second->getType()->getElementType() == llvm::Type::getFloatTy(d->module->getContext()) )
     {
-        GTL_DEBUG("Set " << _value.asFloat() << " on ptr " << ptr << " from value = " << *(float*)ptr);
-        *(float*)ptr = _value.asFloat();
+        GTL_DEBUG("Set " << _value.asFloat32() << " on ptr " << ptr << " from value = " << *(float*)ptr);
+        *(float*)ptr = _value.asFloat32();
         GTL_DEBUG( *(float*)ptr );
     }
   } else {
@@ -532,7 +551,7 @@ GTLCore::Value Program::varying( const GTLCore::String& _name ) const
     void* ptr = it->second;
     if( it2->second->getType()->getElementType() == llvm::Type::getInt32Ty(d->module->getContext()) )
     {
-        return GTLCore::Value( *(int*)ptr);
+        return GTLCore::Value( *(gtl_int32*)ptr);
     }
     if( it2->second->getType()->getElementType() == llvm::Type::getInt1Ty(d->module->getContext()) )
     {
